import { FunctionValueGenerator, ValueGenerator } from "./functions/ValueGenerator";
import { Behavior } from "./behaviors/Behavior";
import { Particle } from "./Particle";
import { MetaData, ParticleEmitter } from "./ParticleEmitter";
import { EmitterShape, ShapeJSON } from "./shape/EmitterShape";
import { BaseEvent, BufferGeometry, Layers, Material, Matrix4, Texture, Vector3 } from "three";
import { ColorGenerator, FunctionColorGenerator, FunctionJSON } from "./functions";
import { VFXBatchSettings, RenderMode } from "./VFXBatch";
import { BatchedRenderer } from "./BatchedRenderer";
import { RotationGenerator } from "./functions/RotationGenerator";
export interface BurstParameters {
    time: number;
    count: number;
    cycle: number;
    interval: number;
    probability: number;
}
export interface ParticleSystemParameters {
    autoDestroy?: boolean;
    looping?: boolean;
    prewarm?: boolean;
    duration?: number;
    shape?: EmitterShape;
    startLife?: ValueGenerator | FunctionValueGenerator;
    startSpeed?: ValueGenerator | FunctionValueGenerator;
    startRotation?: ValueGenerator | FunctionValueGenerator | RotationGenerator;
    startSize?: ValueGenerator | FunctionValueGenerator;
    startLength?: ValueGenerator | FunctionValueGenerator;
    startColor?: ColorGenerator | FunctionColorGenerator;
    emissionOverTime?: ValueGenerator | FunctionValueGenerator;
    emissionOverDistance?: ValueGenerator | FunctionValueGenerator;
    emissionBursts?: Array<BurstParameters>;
    onlyUsedByOther?: boolean;
    behaviors?: Array<Behavior>;
    instancingGeometry?: BufferGeometry;
    renderMode?: RenderMode;
    rendererEmitterSettings?: TrailSettings | MeshSettings | BillBoardSettings;
    speedFactor?: number;
    material: Material;
    layers?: Layers;
    startTileIndex?: ValueGenerator;
    uTileCount?: number;
    vTileCount?: number;
    renderOrder?: number;
    worldSpace?: boolean;
}
export interface ParticleSystemJSONParameters {
    version: string;
    autoDestroy: boolean;
    looping: boolean;
    prewarm: boolean;
    duration: number;
    shape: ShapeJSON;
    startLife: FunctionJSON;
    startSpeed: FunctionJSON;
    startRotation: FunctionJSON;
    startSize: FunctionJSON;
    startColor: FunctionJSON;
    emissionOverTime: FunctionJSON;
    emissionOverDistance: FunctionJSON;
    emissionBursts?: Array<BurstParameters>;
    onlyUsedByOther: boolean;
    rendererEmitterSettings: {
        startLength?: FunctionJSON;
        followLocalOrigin?: boolean;
    };
    instancingGeometry?: any;
    renderMode: number;
    renderOrder?: number;
    speedFactor?: number;
    texture?: string;
    material: string;
    layers?: number;
    startTileIndex: FunctionJSON | number;
    uTileCount: number;
    vTileCount: number;
    blending?: number;
    transparent?: boolean;
    behaviors: Array<any>;
    worldSpace: boolean;
}
export interface JsonMetaData {
    textures: {
        [uuid: string]: Texture;
    };
    geometries: {
        [uuid: string]: BufferGeometry;
    };
}
export interface BillBoardSettings {
}
export interface TrailSettings {
    startLength: ValueGenerator | FunctionValueGenerator;
    followLocalOrigin: boolean;
}
export interface MeshSettings {
    rotationAxis?: Vector3;
    startRotationX: ValueGenerator | FunctionValueGenerator;
    startRotationY: ValueGenerator | FunctionValueGenerator;
    startRotationZ: ValueGenerator | FunctionValueGenerator;
}
export interface EmissionState {
    burstIndex: number;
    burstWaveIndex: number;
    time: number;
    waitEmiting: number;
}
export interface SerializationOptions {
    useUrlForImage?: boolean;
}
/**
 * ParticleSystem represents a system that generates and controls particles with similar attributes.
 *
 * @class
 */
export declare class ParticleSystem {
    /**
     * Determines whether the ParticleSystem should be automatically disposed when it finishes emitting particles.
     *
     * @type {boolean}
     */
    autoDestroy: boolean;
    /**
     * Determines whether a looping ParticleSystem should prewarm, i.e., the Particle System looks like it has already simulated for one loop when first becoming visible.
     *
     * @type {boolean}
     */
    prewarm: boolean;
    /**
     * Determines whether the ParticleSystem should loop, i.e., restart emitting particles after the duration of the particle system is expired.
     *
     * @type {boolean}
     */
    looping: boolean;
    /**
     * The duration of the ParticleSystem in seconds.
     *
     * @type {number}
     */
    duration: number;
    /**
     * The value generator or function value generator for the starting life of particles.
     *
     * @type {ValueGenerator | FunctionValueGenerator}
     */
    startLife: ValueGenerator | FunctionValueGenerator;
    /**
     * The value generator or function value generator for the starting speed of particles.
     *
     * @type {ValueGenerator | FunctionValueGenerator}
     */
    startSpeed: ValueGenerator | FunctionValueGenerator;
    /**
     * The value generator or function value generator or rotation generator for the starting rotation of particles.
     *
     * @type {ValueGenerator | FunctionValueGenerator | RotationGenerator}
     */
    startRotation: ValueGenerator | FunctionValueGenerator | RotationGenerator;
    /**
     * The value generator or function value generator for the starting size of particles.
     *
     * @type {ValueGenerator | FunctionValueGenerator}
     */
    startSize: ValueGenerator | FunctionValueGenerator;
    /**
     * The color generator or function color generator for the starting color of particles.
     *
     * @type {ColorGenerator | FunctionColorGenerator}
     */
    startColor: ColorGenerator | FunctionColorGenerator;
    /**
     * The value generator for the starting tile index of particles.
     *
     * @type {ValueGenerator}
     */
    startTileIndex: ValueGenerator;
    /**
     * The renderer emitter settings for the ParticleSystem.
     *
     * @type {TrailSettings | MeshSettings | BillBoardSettings}
     */
    rendererEmitterSettings: TrailSettings | MeshSettings | BillBoardSettings;
    /**
     * The value generator or function value generator for the emission rate of particles over time.
     *
     * @type {ValueGenerator | FunctionValueGenerator}
     */
    emissionOverTime: ValueGenerator | FunctionValueGenerator;
    /**
     * The value generator or function value generator for the emission rate of particles over distance.
     *
     * @type {ValueGenerator | FunctionValueGenerator}
     */
    emissionOverDistance: ValueGenerator | FunctionValueGenerator;
    /**
     * An array of burst parameters for the ParticleSystem.
     *
     * @type {Array<BurstParameters>}
     */
    emissionBursts: Array<BurstParameters>;
    /**
     * Determines whether the ParticleSystem is only used by other ParticleSystems.
     *
     * @type {boolean}
     */
    onlyUsedByOther: boolean;
    /**
     * Determines whether the ParticleSystem is in world space or local space.
     *
     * @type {boolean}
     */
    worldSpace: boolean;
    /**
     * In render mode StretchedBillBoard
     * how stretched the particle is in the direction of the camera based on the speed of the particle.
     *
     * @type {number}
     */
    speedFactor: number;
    /**
     * The number of particles in the ParticleSystem.
     *
     * @type {number}
     */
    particleNum: number;
    /**
     * Determines whether the ParticleSystem is paused.
     *
     * @type {boolean}
     */
    paused: boolean;
    /**
     * All the particles in the ParticleSystem.
     *
     * @type {Array<Particle>}
     */
    particles: Array<Particle>;
    /**
     * the shape of the emitter.
     *
     * @type {EmitterShape}
     */
    emitterShape: EmitterShape;
    /**
     * the emitter object that should be added in the scene.
     *
     * @type {ParticleEmitter<BaseEvent>}
     */
    emitter: ParticleEmitter<BaseEvent>;
    /**
     * the VFX renderer settings for the batch renderer
     *
     * @type {VFXBatchSettings}
     */
    rendererSettings: VFXBatchSettings;
    /**
     * whether needs to update renderer settings for the batch renderer
     *
     * @type {boolean}
     */
    neededToUpdateRender: boolean;
    /**
     * a list of particle behaviors in the particle system
     *
     * @type {Array<Behavior>}
     */
    behaviors: Array<Behavior>;
    private prewarmed;
    private emissionState;
    private emitEnded;
    private markForDestroy;
    private previousWorldPos?;
    private temp;
    private travelDistance;
    private normalMatrix;
    /** @internal **/
    _renderer?: BatchedRenderer;
    set time(time: number);
    get time(): number;
    get layers(): Layers;
    get texture(): Texture;
    set texture(texture: Texture);
    get material(): Material;
    set material(material: Material);
    get uTileCount(): number;
    set uTileCount(u: number);
    get vTileCount(): number;
    set vTileCount(v: number);
    get instancingGeometry(): BufferGeometry;
    set instancingGeometry(geometry: BufferGeometry);
    get renderMode(): RenderMode;
    set renderMode(renderMode: RenderMode);
    get renderOrder(): number;
    set renderOrder(renderOrder: number);
    get blending(): import("three").Blending;
    set blending(blending: import("three").Blending);
    constructor(parameters: ParticleSystemParameters);
    pause(): void;
    play(): void;
    private spawn;
    endEmit(): void;
    dispose(): void;
    restart(): void;
    private update;
    emit(delta: number, emissionState: EmissionState, emitterMatrix: Matrix4): void;
    toJSON(meta: MetaData, options?: SerializationOptions): ParticleSystemJSONParameters;
    static fromJSON(json: ParticleSystemJSONParameters, meta: {
        textures: {
            [uuid: string]: Texture;
        };
        materials: {
            [uuoid: string]: Material;
        };
        geometries: {
            [uuid: string]: BufferGeometry;
        };
    }, dependencies: {
        [uuid: string]: Behavior;
    }): ParticleSystem;
    addBehavior(behavior: Behavior): void;
    getRendererSettings(): VFXBatchSettings;
    clone(): ParticleSystem;
}
